using System;
using System.Data;
using System.Collections;

namespace UltraGrid_ExcelExportMulti
{
	/// <summary>
	/// Creates a DataSet to be used for testing purposes.
	/// The DataSet can contain up to three related tables:
	/// customers, orders, and products.
	/// </summary>
	public sealed class DataMaker
	{
		#region Data

		private static DataTable tblCust;
		private static DataTable tblOrd;
		private static DataTable tblProd;
		private static ArrayList ordIDs  = new ArrayList();	
		private static Random    rnd     = new Random();

		#endregion Data

		#region Static Ctor

		static DataMaker()
		{
			CreateTables();
		}

		#endregion Static Ctor

		#region Public Properties

		/// <summary>
		/// Gets a DataSet with one table.
		/// </summary>
		public static DataSet OneBand
		{
			get
			{
				CreateTables();
				DataSet ds = new DataSet( "CustInfo" );
				ds.Tables.Add( tblCust );
				return ds;
			}
		}

		/// <summary>
		/// Gets a DataSet with two tables.
		/// There is a relationship between customers.custID and orders.custID.
		/// </summary>
		public static DataSet TwoBands
		{
			get
			{
				DataSet ds = OneBand;
				ds.Tables.Add( tblOrd );
				DataRelation rel = new DataRelation(
					"custID_custID",
					ds.Tables[ "customers" ].Columns[ "custID" ],
					ds.Tables[ "orders" ].Columns[ "custID" ],
					false
					);
				ds.Relations.Add( rel );
				return ds;
			}
		}

		/// <summary>
		/// Gets a DataSet with three tables.
		/// There is a relationship between customers.custID and orders.custID and
		/// between orders.prodID and products.prodID.
		/// </summary>
		public static DataSet ThreeBands
		{
			get
			{
				DataSet ds = TwoBands;
				ds.Tables.Add( tblProd );
				DataRelation rel = new DataRelation(
					"prodID_prodID",
					ds.Tables[ "orders" ].Columns[ "prodID" ],
					ds.Tables[ "products" ].Columns[ "prodID" ],
					false
					);
				ds.Relations.Add( rel );
				return ds;
			}
		}

		#endregion Public Properties

		#region Private Helpers

		#region CreateTables

		private static void CreateTables()
		{
			CreateCustomersTable();
			FillCustomersTable();

			CreateOrdersTable();
			FillOrdersTable();

			CreateProductsTable();
			FillProductsTable();
		}

		#endregion CreateTables

		#region Customers Table 

		private static void CreateCustomersTable()
		{
			tblCust = new DataTable( "customers" );
			DataColumn col;

			col = new DataColumn( "custID", typeof( int ) );
			tblCust.Columns.Add( col );

			tblCust.PrimaryKey = new DataColumn[] { col };

			col = new DataColumn( "fname", typeof( string ) );
			tblCust.Columns.Add( col );

			col = new DataColumn( "lname", typeof( string ) );
			tblCust.Columns.Add( col );		
		
			col = new DataColumn( "firstContact", typeof( DateTime ) );
			tblCust.Columns.Add( col );
		}

		private static void FillCustomersTable()
		{
			string[] names = {
								"Mike Finnigan",
								"Claudia Vanst",
								"Paul Forrest",
								"Dirk Spiggler",
								"Abraham Lincoln"
							};
			DateTime[] dates = {
								   DateTime.Now.AddYears( -2 ),
								   DateTime.Now.AddMonths( -1 ).AddMinutes( 19 ).AddSeconds( 2 ),
								   DateTime.Now.AddHours( -4 ).AddMinutes( 3 ).AddSeconds( 7 ),
								   DateTime.Now.AddDays( -1 ).AddMinutes( 123 ).AddSeconds( 12 ),
								   DateTime.Now.AddMonths( -1 ).AddMinutes( 19 ).AddSeconds( 3 )
							   };
			DataRow  row;
			char[]   delim = { ' ' };
			string[] name;
			for( int idx = 0; idx < names.Length; ++idx )
			{
				row = tblCust.NewRow();
				name = names[ idx ].Split( delim );

				row[ "custID" ] = idx + 1;				
				row[ "fname" ] = name[ 0 ];
				row[ "lname" ] = name[ 1 ];
				row[ "firstContact"] = dates[idx];

				tblCust.Rows.Add( row );
			}
		}

		#endregion Customers Table

		#region Orders Table 

		private static void CreateOrdersTable()
		{
			tblOrd = new DataTable( "orders" );
			DataColumn col;

			col = new DataColumn( "ordID", typeof( int ) );
			tblOrd.Columns.Add( col );

			tblOrd.PrimaryKey = new DataColumn[] { col };

			col = new DataColumn( "custID", typeof( int ) );
			tblOrd.Columns.Add( col );

			col = new DataColumn( "prodID", typeof( int ) );
			tblOrd.Columns.Add( col );

			col = new DataColumn( "quantity", typeof( int ) );
			tblOrd.Columns.Add( col );

			col = new DataColumn( "shipped", typeof( bool ) );
			tblOrd.Columns.Add( col );
		}

		private static void FillOrdersTable()
		{
			DataRow ordRow;
			foreach( DataRow custRow in tblCust.Rows )
			{
				int iters = rnd.Next( 1, 5 );
				for( int i = 0; i < iters; ++i )
				{
					ordRow = tblOrd.NewRow();

					ordRow[ "ordID" ]    = NewOrderID;
					ordRow[ "custID" ]   = Convert.ToInt32( custRow[ "custID" ] );
					ordRow[ "prodID" ]   = rnd.Next( 1, 6 );
					ordRow[ "quantity" ] = rnd.Next( 1, 200 );
					ordRow[ "shipped" ]  = rnd.Next( 1001 ) < 500 ? true : false;

					tblOrd.Rows.Add( ordRow );
				}
			}
		}
		
		private static int NewOrderID
		{
			get
			{				
				int id = -1;
				while( id == -1 || ordIDs.Contains( id ) )
				{	
					id = rnd.Next( 9999 );
				}
				ordIDs.Add( id );
				return id;
			}
		}

		#endregion Orders Table
	
		#region Products Table 

		private static void CreateProductsTable()
		{
			tblProd = new DataTable( "products" );
			DataColumn col;
			
			col = new DataColumn( "prodID", typeof( int ) );
			tblProd.Columns.Add( col );

			tblProd.PrimaryKey = new DataColumn[] { col };

			col = new DataColumn( "name", typeof( string ) );
			tblProd.Columns.Add( col );

			col = new DataColumn( "numInStock", typeof( int ) );
			tblProd.Columns.Add( col );
		}

		private static void FillProductsTable()
		{
			string[] names = {
								"Wonder Gizmo",
								"Aunt Betsy's Secret Recipe Book",
								"Sparkling Cat Leash",
								"Braintwister",
								"Doug's Super Hot Sauce"
							 };
			DataRow prodRow;		
			for( int prodID = 0; prodID < names.Length; ++prodID ) 
			{
				prodRow = tblProd.NewRow();		
				
				prodRow[ "prodID" ] = prodID + 1;
				prodRow[ "name" ] = names[ prodID ];
				prodRow[ "numInStock" ] = rnd.Next( 9999 );

				tblProd.Rows.Add( prodRow );
			}
		}

		#endregion Products Table

		#endregion Private Helpers
	}
}
